//----------------------------------------------------------------------------------
// File:   DX10FeatureDemo.cpp
// Author: Andrew Tatarinov
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

//#include <windows.h>

#pragma warning( disable: 4995 )

#include <DXUT.h>
#include <d3d10.h>
#include <d3dx10.h>
#include <math.h>
#include <vector>

#include <DXUTenum.h>
#include <DXUTgui.h>
#include <DXUTsettingsdlg.h>
#include <DXUTres.h>
#include <SDKmisc.h>

#include "resource.h"

// Defines

#define M_PI 3.14159265358979323846f

// Global variables
// DX10 device and rendering related variables

ID3D10Texture2D * g_pCubeMapDepth = NULL;
ID3D10Texture2D * g_pAtlasDepth = NULL;
ID3D10DepthStencilView * g_pCubeMapDepthView = NULL;
ID3D10DepthStencilView * g_pAtlasDepthView = NULL;
ID3D10Effect * g_pEffect = NULL;
ID3D10EffectTechnique * g_pTorusTechnique = NULL;
ID3D10EffectTechnique * g_pBoxTechnique = NULL;
ID3D10InputLayout * g_pTorusVertexLayout = NULL;
ID3D10InputLayout * g_pBoxVertexLayout = NULL;

// DX10 buffers

ID3D10Buffer * g_pTorusIndexBuffer = NULL;
ID3D10Buffer * g_pTorusInstanceData = NULL;
ID3D10Buffer * g_pBoxVertexBuffer = NULL;
ID3D10Buffer * g_pBoxTexCoordBuffer = NULL;
ID3D10Buffer * g_pBoxRealIndexBuffer = NULL;
ID3D10Buffer * g_pBoxIndexBuffer = NULL;

// Shader variables and matrices

ID3D10EffectMatrixVariable * g_pmViewMatrixVariable = NULL;
ID3D10EffectMatrixVariable * g_pmProjectionMatrixVariable = NULL;
ID3D10EffectMatrixVariable * g_pmInverseViewMatrixVariable = NULL;
ID3D10EffectMatrixVariable * g_pmInverseProjectionMatrixVariable = NULL;
ID3D10EffectMatrixVariable * g_pmCubeViewMatrixVariable = NULL;
ID3D10EffectMatrixVariable * g_pmCubeProjMatrixVariable = NULL;
ID3D10EffectVectorVariable * g_pvEyePositionVariable = NULL;
ID3D10EffectVectorVariable * g_pvLightPositionVariable = NULL;
ID3D10EffectVectorVariable * g_pvLightColorVariable = NULL;
ID3D10EffectVectorVariable * g_pvAmbientColorVariable = NULL;
ID3D10EffectScalarVariable * g_pfParallelSlicesVariable = NULL;
ID3D10EffectScalarVariable * g_pfMeridianSlicesVariable = NULL;
ID3D10EffectScalarVariable * g_pfTorusKnotPVariable = NULL;
ID3D10EffectScalarVariable * g_pfTorusKnotQVariable = NULL;
ID3D10EffectScalarVariable * g_pfTime = NULL;
D3DXMATRIX g_pmWorldMatrices [2];
D3DXMATRIX g_mBoxWorldMatrix;
D3DXMATRIX g_mViewMatrix;
D3DXMATRIX g_mProjectionMatrix;
D3DXMATRIX g_pmCubeViewMatrices [6];
D3DXMATRIX g_mCubeProjectionMatrix;

// Textures and related shader resource views

ID3D10Texture2D * g_pDiffTextureArray = NULL;
ID3D10Texture2D * g_pBumpTextureArray = NULL;
ID3D10Texture2D * g_pNoiseTexture = NULL;
ID3D10ShaderResourceView * g_pDiffTextureArrayRV = NULL;
ID3D10ShaderResourceView * g_pBumpTextureArrayRV = NULL;
ID3D10ShaderResourceView * g_pCubeMapTextureRV = NULL;
ID3D10ShaderResourceView * g_pAtlasTextureRV = NULL;
ID3D10ShaderResourceView * g_pBoxVertexBufferRV = NULL;
ID3D10ShaderResourceView * g_pBoxTexCoordBufferRV = NULL;
ID3D10ShaderResourceView * g_pPermTextureSRV = NULL;
ID3D10EffectShaderResourceVariable * g_pDiffTextureVariable = NULL;
ID3D10EffectShaderResourceVariable * g_pBumpTextureVariable = NULL;
ID3D10EffectShaderResourceVariable * g_pCubeMapTextureVariable = NULL;
ID3D10EffectShaderResourceVariable * g_pAtlasTextureVariable = NULL;
ID3D10EffectShaderResourceVariable * g_pBoxVertexBufferVariable = NULL;
ID3D10EffectShaderResourceVariable * g_pBoxTexCoordBufferVariable = NULL;
ID3D10EffectShaderResourceVariable * g_pPermTextureVariable = NULL;

// GUI related variables

CDXUTDialogResourceManager g_DialogResourceManager;
CD3DSettingsDlg g_SettingsDlg;
CDXUTDialog g_HUD;
CDXUTTextHelper * g_pTextHelper = NULL;
ID3DX10Font * g_pTextFont = NULL;       
ID3DX10Sprite * g_pTextSprite = NULL;

// Window dimensions

int g_WindowSizeX = 800;
int g_WindowSizeY = 600;

// Sizes of some vertex and index buffers

int g_TorusIndexCount = 0;
int g_BoxVertexCount = 0;
int g_BoxIndexCount = 0;

// Constants for sphere generation

const int	g_ParallelSlices = 256;
const int	g_MeridianSlices = 16;

// Text toggler

bool g_bRenderText = false;

// Box half size

const float	g_BoxHalfSize = 30.0f;

// Shadowmap sizes

const int g_CubeMapSize = 800;

// Some structures

struct SSimpleVertex
	{
	unsigned int myMultiIndex [2];
	};

struct SInstanceData
	{
	D3DXMATRIX myWorldMatrix;
	};

//--------------------------------------------------------------------------------------
// UI control IDs
//--------------------------------------------------------------------------------------

#define IDC_TOGGLEFULLSCREEN        1
#define IDC_TOGGLEREF               2
#define IDC_CHANGEDEVICE            3
#define IDC_SHADOWMAPTYPE           4

//--------------------------------------------------------------------------------------
int LoadTextureArray( ID3D10Device* pd3dDevice, LPCWSTR* texNames, int texCount, ID3D10Texture2D** tex, ID3D10ShaderResourceView** texRV )
{
    HRESULT hr = S_OK;
    (* tex) = NULL;

    D3D10_TEXTURE2D_DESC desc;
    ZeroMemory( &desc, sizeof( D3D10_TEXTURE2D_DESC ) );

    for( int i = 0; i < texCount; i ++ )
    {
        ID3D10Resource * pRes = NULL;

        wchar_t str [MAX_PATH];
        if( FAILED( hr = DXUTFindDXSDKMediaFileCch( str, MAX_PATH, texNames[i] ) ) )
        {
            return hr;
        }

        D3DX10_IMAGE_LOAD_INFO loadInfo;
        ZeroMemory( &loadInfo, sizeof( D3DX10_IMAGE_LOAD_INFO ) );
        loadInfo.Width = D3DX10_FROM_FILE;
        loadInfo.Height = D3DX10_FROM_FILE;
        loadInfo.Depth = D3DX10_FROM_FILE;
        loadInfo.FirstMipLevel = 0;
        loadInfo.MipLevels = 1;
        loadInfo.Usage = D3D10_USAGE_STAGING;
        loadInfo.BindFlags = 0;
        loadInfo.CpuAccessFlags = D3D10_CPU_ACCESS_WRITE | D3D10_CPU_ACCESS_READ;
        loadInfo.MiscFlags = 0;
        loadInfo.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
        loadInfo.Filter = D3DX10_FILTER_NONE;
        loadInfo.MipFilter = D3DX10_FILTER_NONE;

        if( FAILED( D3DX10CreateTextureFromFile( pd3dDevice, str, &loadInfo, NULL, &pRes, &hr ) ) || (!pRes) )
        {
            DXUTTRACE( L"Error while creating a texture from file %s\n", texNames [i] );
            return false;
        }

        ID3D10Texture2D * pTemp;
        pRes->QueryInterface( __uuidof( ID3D10Texture2D ), (LPVOID *)&pTemp );
        pTemp->GetDesc( &desc );

        D3D10_MAPPED_TEXTURE2D mappedTex2D;
        if( DXGI_FORMAT_R8G8B8A8_UNORM != desc.Format )
        {
            return false;
        }

        if( (* tex) == NULL )
        {
            desc.Usage = D3D10_USAGE_DEFAULT;
            desc.BindFlags = D3D10_BIND_SHADER_RESOURCE;
            desc.CPUAccessFlags = 0;
            desc.ArraySize = texCount;

            if( FAILED( pd3dDevice->CreateTexture2D( &desc, NULL, tex ) ) )
            {
                return false;
            }
        }

        for( unsigned int j = 0; j < desc.MipLevels; j ++ )
        {
            pTemp->Map( j, D3D10_MAP_READ, 0, &mappedTex2D );

            pd3dDevice->UpdateSubresource( (* tex), D3D10CalcSubresource( j, i, desc.MipLevels ), NULL,
				            mappedTex2D.pData, mappedTex2D.RowPitch, 0 );

            pTemp->Unmap(j);
        }

        pRes->Release();
        pTemp->Release();
    }

    D3D10_SHADER_RESOURCE_VIEW_DESC SRVDesc;

    ZeroMemory( &SRVDesc, sizeof( SRVDesc ) );
    SRVDesc.Format = desc.Format;
    SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2DARRAY;
    SRVDesc.Texture2DArray.MipLevels = desc.MipLevels;
    SRVDesc.Texture2DArray.ArraySize = texCount;

    if( FAILED( pd3dDevice->CreateShaderResourceView( (* tex), &SRVDesc, texRV ) ) )
    {
        return false;
    }

    return true;
}

//--------------------------------------------------------------------------------------
int PrepareCubeMap( ID3D10Device* pd3dDevice )
{
    // Create cubic depth stencil texture.

    D3D10_TEXTURE2D_DESC dstex;

    dstex.Width = g_CubeMapSize;
    dstex.Height = g_CubeMapSize;
    dstex.MipLevels = 1;
    dstex.ArraySize = 6;
    dstex.SampleDesc.Count = 1;
    dstex.SampleDesc.Quality = 0;
    dstex.Format = DXGI_FORMAT_R24G8_TYPELESS;
    dstex.Usage = D3D10_USAGE_DEFAULT;
    dstex.BindFlags = D3D10_BIND_DEPTH_STENCIL | D3D10_BIND_SHADER_RESOURCE;
    dstex.CPUAccessFlags = 0;
    dstex.MiscFlags = 0;

    if( FAILED( pd3dDevice->CreateTexture2D( &dstex, NULL, &g_pCubeMapDepth ) ) )
    {
        DXUTTRACE( L"Failed to create depth stencil texture\n" );
        return false;
    }

    // Create the depth stencil view for the entire cube

    D3D10_DEPTH_STENCIL_VIEW_DESC DescDS;

    DescDS.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
    DescDS.ViewDimension = D3D10_DSV_DIMENSION_TEXTURE2DARRAY;
    DescDS.Texture2DArray.FirstArraySlice = 0;
    DescDS.Texture2DArray.ArraySize = 6;
    DescDS.Texture2DArray.MipSlice = 0;

    if( FAILED( pd3dDevice->CreateDepthStencilView( g_pCubeMapDepth, &DescDS, &g_pCubeMapDepthView ) ) )
    {
        DXUTTRACE( L"Failed to create depth stencil view for a depth cube map\n" );
        return false;
    }

    // Create the shader resource view for the shadow map

    D3D10_SHADER_RESOURCE_VIEW_DESC SRVDesc;

    ZeroMemory( &SRVDesc, sizeof( SRVDesc ) );

    SRVDesc.Format = DXGI_FORMAT_R24_UNORM_X8_TYPELESS;
    SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2DARRAY;
    SRVDesc.Texture2DArray.MipLevels = 1;
    SRVDesc.Texture2DArray.MostDetailedMip = 0;
    SRVDesc.Texture2DArray.FirstArraySlice = 0;
    SRVDesc.Texture2DArray.ArraySize = 6;

    if( FAILED( pd3dDevice->CreateShaderResourceView( g_pCubeMapDepth, &SRVDesc, &g_pCubeMapTextureRV ) ) )
    {
        DXUTTRACE( L"Failed to create shader resource view for a depth stencil\n" );
        return false;
    }

    return true;
}

//--------------------------------------------------------------------------------------
int ReleaseCubeMap()
{
    SAFE_RELEASE( g_pCubeMapDepth );
    SAFE_RELEASE( g_pCubeMapDepthView );
    SAFE_RELEASE( g_pCubeMapTextureRV );

    return true;
}

//--------------------------------------------------------------------------------------
void InitTorusMatrices( ID3D10Device* pd3dDevice )
{
    // Set matrices
    // Initialize the world matrices

    D3DXMatrixIdentity( &g_pmWorldMatrices[0] );
    D3DXMatrixIdentity( &g_pmWorldMatrices[1] );
    D3DXMatrixIdentity( &g_mBoxWorldMatrix );

    // Initialize the projection matrix

    D3DXMatrixPerspectiveFovLH( &g_mProjectionMatrix, (float)D3DX_PI * 0.25f, g_WindowSizeX / (float)g_WindowSizeY, 1.0f, 1000.0f );

    // Set the world matrices to a buffer

    D3D10_BUFFER_DESC bufferDesc =
    {
        sizeof( SInstanceData ) * 2,
        D3D10_USAGE_DYNAMIC,
        D3D10_BIND_VERTEX_BUFFER,
        D3D10_CPU_ACCESS_WRITE,
        0
    };

    if( FAILED( pd3dDevice->CreateBuffer( &bufferDesc, NULL, &g_pTorusInstanceData ) ) )
    {
        return;
    }

    SInstanceData * pInstances = NULL;
    if( FAILED( g_pTorusInstanceData->Map( D3D10_MAP_WRITE_DISCARD, NULL, (void **)&pInstances ) ) || !pInstances )
	{
        return;
	}

    memcpy( pInstances, g_pmWorldMatrices, sizeof( SInstanceData ) * 2 );

    g_pTorusInstanceData->Unmap();
}

//--------------------------------------------------------------------------------------
void SetViewMatrix(bool isPaused)
{
    D3DXMATRIX spinY, spinZ;

	if (!isPaused)
	{
		D3DXMatrixRotationY( &spinY, 0.02f );
		D3DXMatrixRotationZ( &spinZ, 0.01f );
		D3DXMatrixMultiply( &g_mBoxWorldMatrix, &g_mBoxWorldMatrix, &spinZ );
		D3DXMatrixMultiply( &g_mBoxWorldMatrix, &g_mBoxWorldMatrix, &spinY );
	}

    // Initialize the view matrix

    D3DXVECTOR4 Eye( 8.0f, -12.0f, 22.0f, 0.0f );
    D3DXVECTOR3 At( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 Up( 0.0f, 1.0f, 0.0f );

    D3DXVec3Transform( (D3DXVECTOR4 *) &Eye, (D3DXVECTOR3 *) &Eye, &g_mBoxWorldMatrix );

    D3DXMatrixLookAtLH( &g_mViewMatrix, (D3DXVECTOR3 *) &Eye, &At, &Up );
    
    g_pvEyePositionVariable->SetFloatVector( (float *)&Eye );
}

//--------------------------------------------------------------------------------------
void InitCubeMatrices( float * cubeCenter )
{
    D3DXVECTOR3 vLookDir;
    D3DXVECTOR3 vUpDir;

    vLookDir = D3DXVECTOR3( 1.0f, 0.0f, 0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[0], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    vLookDir = D3DXVECTOR3( -1.0f, 0.0f, 0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[1], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    vLookDir = D3DXVECTOR3( 0.0f, 1.0f,  0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f, 0.0f, -1.0f );
    D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[2], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    vLookDir = D3DXVECTOR3( 0.0f, -1.0f, 0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f,  0.0f, 1.0f );
    D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[3], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    vLookDir = D3DXVECTOR3( 0.0f, 0.0f, 1.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[4], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    vLookDir = D3DXVECTOR3( 0.0f, 0.0f, -1.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
    vUpDir = D3DXVECTOR3( 0.0f, 1.0f,  0.0f );
    D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[5], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );

    D3DXMatrixPerspectiveFovLH( &g_mCubeProjectionMatrix, (float)D3DX_PI * 0.5f, 1.0f, 1.0f, 1000.0f );
}
	
//--------------------------------------------------------------------------------------
// Create only indexes for mesh - vertices we will generate in vertex shader
//--------------------------------------------------------------------------------------
void CreateOnlyIndexedMesh( unsigned long ** indices, int * indCount )
{
    int indexCount = 0,
    meridianSlices = g_MeridianSlices,
    parallelSlices = g_ParallelSlices;

    (* indCount) = meridianSlices * parallelSlices * 6;

    (* indices) = new unsigned long [(* indCount)];

    for( int i = 0; i < meridianSlices; i ++ )
    {
        for( int j = 0; j < parallelSlices; j ++ )
        {
            (* indices) [indexCount] = (i) * (parallelSlices + 1) + (j);
            indexCount ++;

            (* indices) [indexCount] = (i) * (parallelSlices + 1)+ (j + 1);
            indexCount ++;

            (* indices) [indexCount] = (i + 1) * (parallelSlices + 1) + (j);
            indexCount ++;

            (* indices) [indexCount] = (i + 1) * (parallelSlices + 1) + (j);
            indexCount ++;

            (* indices) [indexCount] = (i) * (parallelSlices + 1) + (j + 1);
            indexCount ++;

            (* indices) [indexCount] = (i + 1) * (parallelSlices + 1) + (j + 1);
            indexCount ++;
        }
    }
}

//--------------------------------------------------------------------------------------
// Create a multi-indexed box
//--------------------------------------------------------------------------------------
void CreateIndexedBox( ID3D10Device* pd3dDevice, SSimpleVertex ** vertices, int * vertCount, unsigned long ** indices, int * indCount )
{
    int positionsCount = 8;
    int texCoordsCount = 4;

    (* vertCount) = 24;
    (* indCount) = 36;

    (* vertices) = new SSimpleVertex[(* vertCount)];
    (* indices) = new unsigned long[(* indCount)];

    // Create an array of positions

    D3DXVECTOR4 * positions = new D3DXVECTOR4[positionsCount];
    D3DXVECTOR4 * texCoords = new D3DXVECTOR4[texCoordsCount];

    positions[0] = D3DXVECTOR4( -g_BoxHalfSize, -g_BoxHalfSize, -g_BoxHalfSize, 1.0f ); // 0, 8, 16
    positions[1] = D3DXVECTOR4( g_BoxHalfSize, -g_BoxHalfSize, -g_BoxHalfSize, 1.0f ); // 1, 9, 20
    positions[2] = D3DXVECTOR4( g_BoxHalfSize, -g_BoxHalfSize, g_BoxHalfSize, 1.0f ); // 2, 13, 21
    positions[3] = D3DXVECTOR4( -g_BoxHalfSize, -g_BoxHalfSize, g_BoxHalfSize, 1.0f ); // 3, 12, 17
    positions[4] = D3DXVECTOR4( -g_BoxHalfSize, g_BoxHalfSize, -g_BoxHalfSize, 1.0f ); // 4, 11, 19
    positions[5] = D3DXVECTOR4( g_BoxHalfSize, g_BoxHalfSize, -g_BoxHalfSize, 1.0f ); // 5, 10, 23
    positions[6] = D3DXVECTOR4( g_BoxHalfSize, g_BoxHalfSize, g_BoxHalfSize, 1.0f ); // 6, 14, 22
    positions[7] = D3DXVECTOR4( -g_BoxHalfSize, g_BoxHalfSize, g_BoxHalfSize, 1.0f ); // 7, 15, 18

    // Create a buffer and make a shader resource view

    D3D10_BUFFER_DESC bd;
    D3D10_SUBRESOURCE_DATA InitData;

    bd.Usage = D3D10_USAGE_DEFAULT;
    bd.ByteWidth = sizeof (D3DXVECTOR4) * positionsCount;
    bd.BindFlags = D3D10_BIND_SHADER_RESOURCE;
    bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;

    InitData.pSysMem = positions;
    if( FAILED( pd3dDevice->CreateBuffer( &bd, &InitData, &g_pBoxVertexBuffer ) ) )
    {
        DXUTTRACE( L"Failed to create a vertex buffer for a box\n" );
        return;
    }

    D3D10_SHADER_RESOURCE_VIEW_DESC SRVDesc;

    ZeroMemory( &SRVDesc, sizeof( SRVDesc ) );
    SRVDesc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
    SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_BUFFER;
    SRVDesc.Buffer.ElementOffset = 0;
    SRVDesc.Buffer.ElementWidth = positionsCount;

    if( FAILED( pd3dDevice->CreateShaderResourceView( g_pBoxVertexBuffer, &SRVDesc, &g_pBoxVertexBufferRV ) ) )
    {
        DXUTTRACE( L"Failed to create shader resource view for a box vertex buffer\n" );
        return;
    }

    // Set an array of texture coordinates

    texCoords[0] = D3DXVECTOR4( 0.0f, 0.0f, 0.0f, 0.0f );
    texCoords[1] = D3DXVECTOR4( 1.0f, 0.0f, 0.0f, 0.0f );
    texCoords[2] = D3DXVECTOR4( 1.0f, 1.0f, 0.0f, 0.0f );
    texCoords[3] = D3DXVECTOR4( 0.0f, 1.0f, 0.0f, 0.0f );

    bd.Usage = D3D10_USAGE_DEFAULT;
    bd.ByteWidth = sizeof( D3DXVECTOR4 ) * texCoordsCount;
    bd.BindFlags = D3D10_BIND_SHADER_RESOURCE;
    bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;

    InitData.pSysMem = texCoords;
    if( FAILED( pd3dDevice->CreateBuffer( &bd, &InitData, &g_pBoxTexCoordBuffer ) ) )
    {
        DXUTTRACE( L"Failed to create a texture coordinate buffer for a box\n" );
        return;
    }

    ZeroMemory( &SRVDesc, sizeof( SRVDesc ) );
    SRVDesc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
    SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_BUFFER;
    SRVDesc.Buffer.ElementOffset = 0;
    SRVDesc.Buffer.ElementWidth = texCoordsCount;

    if( FAILED( pd3dDevice->CreateShaderResourceView( g_pBoxTexCoordBuffer, &SRVDesc, &g_pBoxTexCoordBufferRV ) ) )
    {
        DXUTTRACE( L"Failed to create shader resource view for a box texture coordinate buffer\n" );
        return;
    }
    
    (* vertices)[0].myMultiIndex[0] = 0; (* vertices)[0].myMultiIndex[1] = 0;
    (* vertices)[1].myMultiIndex[0] = 1; (* vertices)[1].myMultiIndex[1] = 1;
    (* vertices)[2].myMultiIndex[0] = 2; (* vertices)[2].myMultiIndex[1] = 2;
    (* vertices)[3].myMultiIndex[0] = 3; (* vertices)[3].myMultiIndex[1] = 3;
    (* vertices)[4].myMultiIndex[0] = 4; (* vertices)[4].myMultiIndex[1] = 0;
    (* vertices)[5].myMultiIndex[0] = 5; (* vertices)[5].myMultiIndex[1] = 1;
    (* vertices)[6].myMultiIndex[0] = 6; (* vertices)[6].myMultiIndex[1] = 2;
    (* vertices)[7].myMultiIndex[0] = 7; (* vertices)[7].myMultiIndex[1] = 3;
    (* vertices)[8].myMultiIndex[0] = 0; (* vertices)[8].myMultiIndex[1] = 0;
    (* vertices)[9].myMultiIndex[0] = 1; (* vertices)[9].myMultiIndex[1] = 1;
    (* vertices)[10].myMultiIndex[0] = 5; (* vertices)[10].myMultiIndex[1] = 2;
    (* vertices)[11].myMultiIndex[0] = 4; (* vertices)[11].myMultiIndex[1] = 3;
    (* vertices)[12].myMultiIndex[0] = 3; (* vertices)[12].myMultiIndex[1] = 0;
    (* vertices)[13].myMultiIndex[0] = 2; (* vertices)[13].myMultiIndex[1] = 1;
    (* vertices)[14].myMultiIndex[0] = 6; (* vertices)[14].myMultiIndex[1] = 2;
    (* vertices)[15].myMultiIndex[0] = 7; (* vertices)[15].myMultiIndex[1] = 3;
    (* vertices)[16].myMultiIndex[0] = 0; (* vertices)[16].myMultiIndex[1] = 0;
    (* vertices)[17].myMultiIndex[0] = 3; (* vertices)[17].myMultiIndex[1] = 1;
    (* vertices)[18].myMultiIndex[0] = 7; (* vertices)[18].myMultiIndex[1] = 2;
    (* vertices)[19].myMultiIndex[0] = 4; (* vertices)[19].myMultiIndex[1] = 3;
    (* vertices)[20].myMultiIndex[0] = 1; (* vertices)[20].myMultiIndex[1] = 0;
    (* vertices)[21].myMultiIndex[0] = 2; (* vertices)[21].myMultiIndex[1] = 1;
    (* vertices)[22].myMultiIndex[0] = 6; (* vertices)[22].myMultiIndex[1] = 2;
    (* vertices)[23].myMultiIndex[0] = 5; (* vertices)[23].myMultiIndex[1] = 3;

    (* indices)[0] = 0;
    (* indices)[1] = 2;
    (* indices)[2] = 1;
    (* indices)[3] = 0;
    (* indices)[4] = 3;
    (* indices)[5] = 2;

    (* indices)[6] = 4;
    (* indices)[7] = 5;
    (* indices)[8] = 6;
    (* indices)[9] = 4;
    (* indices)[10] = 6;
    (* indices)[11] = 7;

    (* indices)[12] = 8;
    (* indices)[13] = 9;
    (* indices)[14] = 10;
    (* indices)[15] = 8;
    (* indices)[16] = 10;
    (* indices)[17] = 11;

    (* indices)[18] = 12;
    (* indices)[19] = 14;
    (* indices)[20] = 13;
    (* indices)[21] = 12;
    (* indices)[22] = 15;
    (* indices)[23] = 14;

    (* indices)[24] = 16;
    (* indices)[25] = 18;
    (* indices)[26] = 17;
    (* indices)[27] = 16;
    (* indices)[28] = 19;
    (* indices)[29] = 18;

    (* indices)[30] = 20;
    (* indices)[31] = 21;
    (* indices)[32] = 22;
    (* indices)[33] = 20;
    (* indices)[34] = 22;
    (* indices)[35] = 23;
}

//--------------------------------------------------------------------------------------
void RotateTorus(bool isPaused)
{
    static float angleX = 0.0f, angleY = 0.0f, angleZ = 0.0f, angleX1 = 0.0f, angleX2 = 0.0f;

	if (!isPaused)
	{
		angleX += 0.04f;
		angleY += 0.02f;
		angleZ += 0.04f;

		angleX1 += 0.1f;
		angleX2 += 0.2f;
	}

    D3DXMatrixIdentity( &g_pmWorldMatrices[0] );
    D3DXMatrixIdentity( &g_pmWorldMatrices[1] );

    D3DXMATRIX spinX, spinY, spinZ, spinX2, translate;

    D3DXMatrixRotationZ( &spinX, angleX );
    D3DXMatrixRotationY( &spinY, angleY );
    D3DXMatrixRotationZ( &spinZ, angleZ );
    D3DXMatrixRotationX( &spinX2, angleX1 );
    D3DXMatrixTranslation( &translate, 6.0f, 0.0f, 0.0f );

    D3DXMatrixMultiply( &g_pmWorldMatrices[0], &g_pmWorldMatrices[0], &spinX2 );
    D3DXMatrixMultiply( &g_pmWorldMatrices[0], &g_pmWorldMatrices[0], &spinX );
    D3DXMatrixMultiply( &g_pmWorldMatrices[0], &g_pmWorldMatrices[0], &translate );
    D3DXMatrixMultiply( &g_pmWorldMatrices[0], &g_pmWorldMatrices[0], &spinY );
    D3DXMatrixMultiply( &g_pmWorldMatrices[0], &g_pmWorldMatrices[0], &spinZ );

    D3DXMatrixRotationX( &spinX2, angleX2 );
    D3DXMatrixTranslation( &translate, -6.0f, 0.0f, 0.0f );

    D3DXMatrixMultiply( &g_pmWorldMatrices[1], &g_pmWorldMatrices[1], &spinX2 );
    D3DXMatrixMultiply( &g_pmWorldMatrices[1], &g_pmWorldMatrices[1], &spinX );
    D3DXMatrixMultiply( &g_pmWorldMatrices[1], &g_pmWorldMatrices[1], &translate );
    D3DXMatrixMultiply( &g_pmWorldMatrices[1], &g_pmWorldMatrices[1], &spinY );
    D3DXMatrixMultiply( &g_pmWorldMatrices[1], &g_pmWorldMatrices[1], &spinZ );
}

//--------------------------------------------------------------------------------------
void RotateLight( float * lightPos, bool isPaused )
{
    D3DXMATRIX spinX, spinY;
    static float lightAngleY = 0.0f, lightAngleX = 0.0f;

	if (!isPaused)
	{
		lightAngleY += 0.03f;
		lightAngleX += 0.05f;
	}

    D3DXMatrixRotationY( &spinY, lightAngleY );
    D3DXMatrixRotationX( &spinX, lightAngleX );
    D3DXMatrixMultiply( &spinY, &spinY, &spinX );
    D3DXVec3Transform( (D3DXVECTOR4 *)lightPos, (D3DXVECTOR3 *)lightPos, &spinY );
}

//--------------------------------------------------------------------------------------
void RenderDemo( ID3D10Device* pd3dDevice )
{
    ID3D10RenderTargetView * RenderTargetView = DXUTGetD3D10RenderTargetView();
    ID3D10DepthStencilView * DepthStencilView = DXUTGetD3D10DepthStencilView();

    SInstanceData * pInstances = NULL;
    g_pTorusInstanceData->Map( D3D10_MAP_WRITE_DISCARD, NULL, (void **)&pInstances );
    memcpy( pInstances, g_pmWorldMatrices, sizeof( SInstanceData ) * 2 );
    g_pTorusInstanceData->Unmap();

    g_pmViewMatrixVariable->SetMatrix( (float *)&g_mViewMatrix );
    g_pmProjectionMatrixVariable->SetMatrix( (float *)&g_mProjectionMatrix );
    
    g_pmCubeViewMatrixVariable->SetMatrixArray( (float *)g_pmCubeViewMatrices, 0, 6 );
    g_pmCubeProjMatrixVariable->SetMatrix( g_mCubeProjectionMatrix );
    g_pfParallelSlicesVariable->SetInt( g_ParallelSlices );
    g_pfMeridianSlicesVariable->SetInt( g_MeridianSlices );
    g_pfTorusKnotPVariable->SetFloat( 3.0f );
    g_pfTorusKnotQVariable->SetFloat( 5.0f );

    // Render torus
    // Set buffers

    unsigned int strides[1] = { sizeof( SInstanceData ) };
    unsigned int offsets[1] = { 0 };
    ID3D10Buffer * pBuffers[1] = { g_pTorusInstanceData };

    pd3dDevice->IASetInputLayout( g_pTorusVertexLayout );
    pd3dDevice->IASetVertexBuffers( 0, 1, pBuffers, strides, offsets );
    pd3dDevice->IASetIndexBuffer( g_pTorusIndexBuffer, DXGI_FORMAT_R32_UINT, 0 );
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

    // Render using current technique

    D3D10_TECHNIQUE_DESC techDesc;

    g_pTorusTechnique->GetDesc( &techDesc );

    // First, render to a cube map
    // Set a new viewport for rendering to cube map

    D3D10_VIEWPORT SMVP;
    
    SMVP.Height		= g_CubeMapSize;
    SMVP.Width		= g_CubeMapSize;
    SMVP.MinDepth	= 0;
    SMVP.MaxDepth	= 1;
    SMVP.TopLeftX	= 0;
    SMVP.TopLeftY	= 0;

    pd3dDevice->RSSetViewports( 1, &SMVP );

    ID3D10RenderTargetView * nullView[] = { NULL };
    pd3dDevice->OMSetRenderTargets( 1, nullView, g_pCubeMapDepthView );

    g_pTorusTechnique->GetPassByIndex(0)->Apply(0);
    pd3dDevice->DrawIndexedInstanced( g_TorusIndexCount, 2, 0, 0, 0 );
    
    // On the second pass, render toruses to color buffer
    
    SMVP.Height = g_WindowSizeY;
    SMVP.Width = g_WindowSizeX;
    SMVP.MinDepth = 0;
    SMVP.MaxDepth = 1;
    SMVP.TopLeftX = 0;
    SMVP.TopLeftY = 0;

    pd3dDevice->RSSetViewports( 1, &SMVP );

    pd3dDevice->OMSetRenderTargets( 1, &RenderTargetView, DepthStencilView );
    g_pTorusTechnique->GetPassByIndex(1)->Apply(0);
    pd3dDevice->DrawIndexedInstanced( g_TorusIndexCount, 2, 0, 0, 0 );

    // Render box
    // Set buffers

    strides [0] = sizeof( SSimpleVertex );
    offsets [0] = 0;
    pBuffers [0] = g_pBoxRealIndexBuffer;

    pd3dDevice->IASetInputLayout( g_pBoxVertexLayout );
    pd3dDevice->IASetVertexBuffers( 0, 1, pBuffers, strides, offsets );
    pd3dDevice->IASetIndexBuffer( g_pBoxIndexBuffer, DXGI_FORMAT_R32_UINT, 0 );
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

    // Render using current technique

    g_pBoxTechnique->GetDesc( &techDesc );
    g_pBoxTechnique->GetPassByIndex(0)->Apply(0);
    pd3dDevice->DrawIndexed( g_BoxIndexCount, 0, 0 );
}

//--------------------------------------------------------------------------------------
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl * pControl, void * pUserContext )
{
	switch( nControlID )
    {
	case IDC_TOGGLEFULLSCREEN: DXUTToggleFullScreen(); break;
    case IDC_TOGGLEREF:        DXUTToggleREF(); break;
    case IDC_CHANGEDEVICE:     g_SettingsDlg.SetActive( !g_SettingsDlg.IsActive() ); break;
    }
}

void InitGUI()
{
    g_SettingsDlg.Init( &g_DialogResourceManager );
    g_HUD.Init( &g_DialogResourceManager );

    g_HUD.SetCallback( OnGUIEvent ); int iY = 10;
    g_HUD.AddButton( IDC_TOGGLEFULLSCREEN, L"Toggle full screen", 35, iY, 125, 22 );
    g_HUD.AddButton( IDC_TOGGLEREF, L"Toggle REF (F3)", 35, iY += 24, 125, 22, VK_F3 );
    g_HUD.AddButton( IDC_CHANGEDEVICE, L"Change device (F2)", 35, iY += 24, 125, 22, VK_F2 );
}

//--------------------------------------------------------------------------------------
// Reject any D3D10 devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Called right before creating a D3D9 or D3D10 device, allowing the app to modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
    // For the first device created if its a REF device, optionally display a warning dialog box
    static bool s_bFirstTime = true;
    if( s_bFirstTime )
    {
        s_bFirstTime = false;
        if( (DXUT_D3D9_DEVICE == pDeviceSettings->ver && pDeviceSettings->d3d9.DeviceType == D3DDEVTYPE_REF) ||
            (DXUT_D3D10_DEVICE == pDeviceSettings->ver && pDeviceSettings->d3d10.DriverType == D3D10_DRIVER_TYPE_REFERENCE) )
            DXUTDisplaySwitchingToREFWarning( pDeviceSettings->ver );
    }

    return true;
}

//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice (ID3D10Device * pd3dDevice, const DXGI_SURFACE_DESC * pBackBufferSurfaceDesc, void * pUserContext)
{
    pd3dDevice = pd3dDevice;

    HRESULT hr = S_OK;

    V_RETURN( g_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( g_SettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( D3DX10CreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, L"Arial", &g_pTextFont ) );
    V_RETURN( D3DX10CreateSprite( pd3dDevice, 512, &g_pTextSprite ) );

    g_pTextHelper = new CDXUTTextHelper( NULL, NULL, g_pTextFont, g_pTextSprite, 15 );

    // Prepare cube map

    PrepareCubeMap( pd3dDevice );

    // Set viewport

    D3D10_VIEWPORT vp;

    vp.Width = g_WindowSizeX;
    vp.Height = g_WindowSizeY;
    vp.MinDepth = 0;
    vp.MaxDepth = 1;
    vp.TopLeftX = 0;
    vp.TopLeftY = 0;

    pd3dDevice->RSSetViewports( 1, &vp );

    // Load effect from file

    wchar_t str [MAX_PATH];
    if( FAILED( hr = DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"Source\\DX10FeatureDemo\\DX10FeatureDemo.fx" ) ) )
    {
        MessageBox( NULL, L"FX file cannot be found.", L"Error", MB_OK );
        return hr;
    }

	ID3D10Blob *pErrors;
    hr = D3DX10CreateEffectFromFile( str, NULL, NULL, "fx_4_0", D3D10_SHADER_ENABLE_STRICTNESS, 0, 
								     pd3dDevice, NULL, NULL, &g_pEffect, &pErrors, &hr );
    if( FAILED( hr ) )
    {
        MessageBoxA (NULL, (char *)pErrors->GetBufferPointer(), "Compilation errors", MB_OK);
        return hr;
    }

    // Obtain techniques

    g_pTorusTechnique = g_pEffect->GetTechniqueByName( "RenderTorus" );	
    g_pBoxTechnique = g_pEffect->GetTechniqueByName( "RenderBox" );

    // Obtain texture variables

    g_pDiffTextureVariable = g_pEffect->GetVariableByName( "DiffTextureArray" )->AsShaderResource();
    g_pBumpTextureVariable = g_pEffect->GetVariableByName( "BumpTextureArray" )->AsShaderResource();
    g_pPermTextureVariable = g_pEffect->GetVariableByName( "PermTexture" )->AsShaderResource();
    g_pCubeMapTextureVariable = g_pEffect->GetVariableByName( "ShadowMap" )->AsShaderResource();
    g_pAtlasTextureVariable = g_pEffect->GetVariableByName( "ShadowAtlas" )->AsShaderResource();
    g_pBoxVertexBufferVariable = g_pEffect->GetVariableByName( "BoxVertexBuffer" )->AsShaderResource();
    g_pBoxTexCoordBufferVariable = g_pEffect->GetVariableByName( "BoxTexCoordBuffer" )->AsShaderResource();

    // Obtain the matrix variables

    g_pmViewMatrixVariable = g_pEffect->GetVariableByName( "ViewMatrix" )->AsMatrix();
    g_pmProjectionMatrixVariable = g_pEffect->GetVariableByName( "ProjectionMatrix" )->AsMatrix();
    g_pmInverseViewMatrixVariable = g_pEffect->GetVariableByName( "InverseViewMatrix" )->AsMatrix();
    g_pmInverseProjectionMatrixVariable = g_pEffect->GetVariableByName( "InverseProjectionMatrix" )->AsMatrix();
    g_pmCubeViewMatrixVariable = g_pEffect->GetVariableByName( "CubeViewMatrices" )->AsMatrix();
    g_pmCubeProjMatrixVariable = g_pEffect->GetVariableByName( "CubeProjectionMatrix" )->AsMatrix();

    // Obtain light variables

    g_pvEyePositionVariable = g_pEffect->GetVariableByName( "EyePosition" )->AsVector();
    g_pvLightPositionVariable = g_pEffect->GetVariableByName( "LightPosition" )->AsVector();
    g_pvLightColorVariable = g_pEffect->GetVariableByName( "LightColor" )->AsVector();
    g_pvAmbientColorVariable = g_pEffect->GetVariableByName( "AmbientColor" )->AsVector();

    // Obtain other variables

    g_pfParallelSlicesVariable = g_pEffect->GetVariableByName( "ParallelSlices" )->AsScalar();
    g_pfMeridianSlicesVariable = g_pEffect->GetVariableByName( "MeridianSlices" )->AsScalar();
    g_pfTorusKnotPVariable = g_pEffect->GetVariableByName( "TorusKnotP" )->AsScalar();
    g_pfTorusKnotQVariable = g_pEffect->GetVariableByName( "TorusKnotQ" )->AsScalar();
    g_pfTime = g_pEffect->GetVariableByName( "Time" )->AsScalar();

    // Set inpit layouts

    D3D10_INPUT_ELEMENT_DESC torusLayout[] =
    {
        { "WorldTransform", 0, DXGI_FORMAT_R32G32B32A32_FLOAT, 0, 0, D3D10_INPUT_PER_INSTANCE_DATA, 1 },
        { "WorldTransform", 1, DXGI_FORMAT_R32G32B32A32_FLOAT, 0, 16, D3D10_INPUT_PER_INSTANCE_DATA, 1 },
        { "WorldTransform", 2, DXGI_FORMAT_R32G32B32A32_FLOAT, 0, 32, D3D10_INPUT_PER_INSTANCE_DATA, 1 },
        { "WorldTransform", 3, DXGI_FORMAT_R32G32B32A32_FLOAT, 0, 48, D3D10_INPUT_PER_INSTANCE_DATA, 1 }
    };

    unsigned int numElements = sizeof( torusLayout ) / sizeof( torusLayout[0] );

    D3D10_PASS_DESC PassDesc;

    g_pTorusTechnique->GetPassByIndex(0)->GetDesc( &PassDesc );
    if( FAILED( hr = pd3dDevice->CreateInputLayout( torusLayout, numElements, PassDesc.pIAInputSignature,
                                                      PassDesc.IAInputSignatureSize, &g_pTorusVertexLayout ) ) )
    {
        DXUTTRACE( L"Error %d while creating input layout for torus\n", HRESULT_CODE( hr ) );
        return hr;
    }

    D3D10_INPUT_ELEMENT_DESC boxLayout[] =
    {
        { "TEXCOORD", 0, DXGI_FORMAT_R32G32_UINT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 }
    };

    numElements = sizeof( boxLayout ) / sizeof( boxLayout[0] );

    g_pBoxTechnique->GetPassByIndex(0)->GetDesc( &PassDesc );
    if( FAILED( hr = pd3dDevice->CreateInputLayout( boxLayout, numElements, PassDesc.pIAInputSignature,
                                                      PassDesc.IAInputSignatureSize, &g_pBoxVertexLayout ) ) )
	    {
	    DXUTTRACE( L"Error %d while creating input layout for box\n", HRESULT_CODE( hr ) );
	    return hr;
	    }

    // Preparing torus indices and setting buffers

    unsigned long * indices;

    CreateOnlyIndexedMesh( &indices, &g_TorusIndexCount );

    D3D10_BUFFER_DESC bd;
    D3D10_SUBRESOURCE_DATA InitData;

    bd.Usage = D3D10_USAGE_DEFAULT;
    bd.ByteWidth = sizeof( unsigned long ) * g_TorusIndexCount;
    bd.BindFlags = D3D10_BIND_INDEX_BUFFER;
    bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;

    InitData.pSysMem = indices;
    if( FAILED( hr = pd3dDevice->CreateBuffer( &bd, &InitData, &g_pTorusIndexBuffer ) ) )
    {
        return hr;
    }

    // Preparing box and setting buffers

    SSimpleVertex * vertices;

    CreateIndexedBox( pd3dDevice, &vertices, &g_BoxVertexCount, &indices, &g_BoxIndexCount );

    bd.Usage = D3D10_USAGE_DEFAULT;
    bd.ByteWidth = sizeof( SSimpleVertex ) * g_BoxVertexCount;
    bd.BindFlags = D3D10_BIND_VERTEX_BUFFER;
    bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;

    InitData.pSysMem = vertices;
    if (FAILED (hr = pd3dDevice->CreateBuffer(&bd, &InitData, &g_pBoxRealIndexBuffer ) ) )
    {
        return hr;
    }

    bd.Usage = D3D10_USAGE_DEFAULT;
    bd.ByteWidth = sizeof( unsigned long ) * g_BoxIndexCount;
    bd.BindFlags = D3D10_BIND_INDEX_BUFFER;
    bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;

    InitData.pSysMem = indices;
    if (FAILED (hr = pd3dDevice->CreateBuffer( &bd, &InitData, &g_pBoxIndexBuffer ) ) )
    {
        return hr;
    }

    // Load textures

    LPCWSTR diffTexNames[] = { L"Media\\Torus1Diff.dds",
                               L"Media\\Torus2Diff.dds" };
    LPCWSTR bumpTexNames[] = { L"Media\\Torus1Bump.dds",
                               L"Media\\Torus2Bump.dds" };

    LoadTextureArray( pd3dDevice, diffTexNames, 2, &g_pDiffTextureArray, &g_pDiffTextureArrayRV );
    LoadTextureArray( pd3dDevice, bumpTexNames, 2, &g_pBumpTextureArray, &g_pBumpTextureArrayRV );

    g_pDiffTextureVariable->SetResource( g_pDiffTextureArrayRV );
    g_pBumpTextureVariable->SetResource( g_pBumpTextureArrayRV );
    g_pCubeMapTextureVariable->SetResource( g_pCubeMapTextureRV );
    g_pAtlasTextureVariable->SetResource( g_pAtlasTextureRV );
    g_pBoxVertexBufferVariable->SetResource( g_pBoxVertexBufferRV );
    g_pBoxTexCoordBufferVariable->SetResource( g_pBoxTexCoordBufferRV );

    // Create noise texture
    // Fill the texture with random numbers from 0 to 256
    
    srand( GetTickCount() );

    BYTE data[256 * 256];
    for (int i = 0; i < 256 * 256; i++)
    {
        data[i] = (unsigned char) (rand () % 256);
    }

    D3D10_TEXTURE2D_DESC desc;
    desc.Width = 256;
    desc.Height = 256;
    desc.MipLevels = 1;
    desc.ArraySize = 1;
    desc.Format = DXGI_FORMAT_R8_TYPELESS;
    desc.SampleDesc.Count = 1;
    desc.SampleDesc.Quality = 0;
    desc.Usage = D3D10_USAGE_IMMUTABLE;
    desc.BindFlags = D3D10_BIND_SHADER_RESOURCE;
    desc.CPUAccessFlags = 0;
    desc.MiscFlags = 0;

    D3D10_SUBRESOURCE_DATA dataDesc;
    dataDesc.pSysMem = data;
    dataDesc.SysMemPitch = 256;

    V( pd3dDevice->CreateTexture2D(&desc, &dataDesc, &g_pNoiseTexture) );

    // Create the shader resource view for jittering
    D3D10_SHADER_RESOURCE_VIEW_DESC descSRV;

    ZeroMemory( &descSRV, sizeof(descSRV) );
    descSRV.Format = DXGI_FORMAT_R8_UINT;
    descSRV.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    descSRV.Texture2D.MipLevels = 1;
    descSRV.Texture2D.MostDetailedMip = 0;

    V( pd3dDevice->CreateShaderResourceView( g_pNoiseTexture, &descSRV, &g_pPermTextureSRV ) );

    g_pPermTextureVariable -> SetResource( g_pPermTextureSRV );
    
    // Initializing matrices and preparing matrix buffers

    InitTorusMatrices( pd3dDevice );

    if( FAILED( hr = g_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) ) ||
	    FAILED( hr = g_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) ) )
    {
        DXUTTRACE (L"g_DialogResourceManager::OnD3D10CreateDevice() failed\n");
        return hr;
    }

    return S_OK;
}
//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device * pd3dDevice, IDXGISwapChain * pSwapChain, const DXGI_SURFACE_DESC * pBackBufferSurfaceDesc, void * pUserContext )
{
    HRESULT hr;

	g_WindowSizeX = pBackBufferSurfaceDesc->Width;
	g_WindowSizeY = pBackBufferSurfaceDesc->Height;
	
    g_HUD.SetLocation( pBackBufferSurfaceDesc->Width - 170, 0);
    g_HUD.SetSize( 170, 170 );

    V_RETURN( g_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );
    V_RETURN( g_SettingsDlg.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );

	return S_OK;
}

//--------------------------------------------------------------------------------------
// Handle updates to the scene.  This is called regardless of which D3D API is used
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
	const bool isPaused = DXUTIsTimePaused();

	// Make transformations and set matrices
    RotateTorus(isPaused);

    D3DXMatrixPerspectiveFovLH( &g_mProjectionMatrix, (float)D3DX_PI * 0.25f, g_WindowSizeX / (float)g_WindowSizeY, 1.0f, 1000.0f );
    SetViewMatrix(isPaused);

    // Set light
    float lightPos[] = { -20.0f, 0.0f, 0.0f, 1.0f };
    float lightCol[] = { 1.0f, 1.0f, 1.0f, 1.0f };
    float ambientCol[] = { 0.2f, 0.2f, 0.2f, 1.0f };

    RotateLight( lightPos, isPaused );
    InitCubeMatrices( lightPos );

    g_pvLightPositionVariable->SetFloatVector( lightPos );
    g_pvLightColorVariable->SetFloatVector( lightCol );
    g_pvAmbientColorVariable->SetFloatVector( ambientCol );
}

//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device * pd3dDevice, double fTime, float fElapsedTime, void * pUserContext )
{
    // Clear buffers

    float ClearColor [4] = { 0.0f, 0.0f, 0.0f, 1.0f };

    ID3D10RenderTargetView * RenderTargetView = DXUTGetD3D10RenderTargetView();
    ID3D10DepthStencilView * DepthStencilView = DXUTGetD3D10DepthStencilView();

    pd3dDevice->ClearRenderTargetView( RenderTargetView, ClearColor );
    pd3dDevice->ClearDepthStencilView( DepthStencilView, D3D10_CLEAR_DEPTH, 1.0f, 0 );
    
    pd3dDevice->ClearDepthStencilView( g_pCubeMapDepthView, D3D10_CLEAR_DEPTH, 1.0f, 0 );
    
    g_pfTime->SetFloat( (float)fTime );
    
    RenderDemo( pd3dDevice );

    // Render GUI

    g_HUD.OnRender( fElapsedTime );
    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.OnRender( fElapsedTime );
        return;
    }

    // Show text

    g_pTextHelper->Begin ();
    g_pTextHelper->SetInsertionPos( 5, 5 );
    g_pTextHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
    g_pTextHelper->DrawTextLine( DXUTGetFrameStats(true) );
    g_pTextHelper->DrawTextLine( DXUTGetDeviceStats() );
    g_pTextHelper->End ();
    
    if( g_bRenderText )
    {
        g_pTextHelper->Begin();
        g_pTextHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 1.0f, 1.0f ) );
        
        g_pTextHelper->SetInsertionPos( 5, g_WindowSizeY - 180 );
        g_pTextHelper->DrawTextLine( L"Direct3D 10 features used:\n"
                                     L"  - Instancing\n"
                                     L"  - Texture arrays\n"
                                     L"  - Vertex texture fetching\n"
                                     L"  - VertexID, InstanceID and PrimitiveID\n"
                                     L"  - Geometry shaders\n"
                                     L"  - Rendering to a texture array\n"
                                     L"  - Integer math\n"
                                     L"  - Vertex buffer binding as shader resource\n"
                                     L"    and 'Load' instruction\n"
                                     L"  - Complex pixel shaders\n" );

        g_pTextHelper->SetInsertionPos( g_WindowSizeX - 250, g_WindowSizeY - 40 );

        g_pTextHelper->End ();
    }
}

//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10ResizedSwapChain 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10ReleasingSwapChain( void * pUserContext )
{
    g_DialogResourceManager.OnD3D10ReleasingSwapChain();

    ID3D10RenderTargetView * pNullView = { NULL };
    DXUTGetD3D10Device()->OMSetRenderTargets( 1, &pNullView, NULL );
}

//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void * pUserContext )
{
    g_DialogResourceManager.OnD3D10DestroyDevice();
    g_SettingsDlg.OnD3D10DestroyDevice();

    SAFE_RELEASE( g_pTorusIndexBuffer );
    SAFE_RELEASE( g_pTorusInstanceData );
    SAFE_RELEASE( g_pBoxVertexBuffer );
    SAFE_RELEASE( g_pBoxTexCoordBuffer );
    SAFE_RELEASE( g_pBoxRealIndexBuffer );
    SAFE_RELEASE( g_pBoxIndexBuffer );

    SAFE_RELEASE( g_pDiffTextureArray );
    SAFE_RELEASE( g_pBumpTextureArray );
    SAFE_RELEASE( g_pDiffTextureArrayRV );
    SAFE_RELEASE( g_pBumpTextureArrayRV );
    SAFE_RELEASE( g_pCubeMapTextureRV );
    SAFE_RELEASE( g_pBoxVertexBufferRV );
    SAFE_RELEASE( g_pBoxTexCoordBufferRV );

    SAFE_RELEASE( g_pCubeMapDepth );
    SAFE_RELEASE( g_pCubeMapDepthView );
    SAFE_RELEASE( g_pEffect );
    SAFE_RELEASE( g_pTorusVertexLayout );
    SAFE_RELEASE( g_pBoxVertexLayout );

    SAFE_DELETE( g_pTextHelper );
    SAFE_RELEASE( g_pTextFont );
    SAFE_RELEASE( g_pTextSprite );
}

//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
                         bool* pbNoFurtherProcessing, void* pUserContext )
{
    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
        return 0;
    }

    *pbNoFurtherProcessing = g_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing == true )
        return 0;

    *pbNoFurtherProcessing = g_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing == true )
        return 0;

    return 0;
}

//--------------------------------------------------------------------------------------
// Handle key presses
//--------------------------------------------------------------------------------------
void CALLBACK KeyboardProc( UINT nChar, bool bKeyDown, bool bAltDown, void * pUserContext )
{
    if( !bKeyDown )
    {
        return;
    }

    switch( nChar )
	{
        case VK_F1:
			g_bRenderText = !g_bRenderText;
			break;
    }
}

//--------------------------------------------------------------------------------------
// Handle mouse button presses
//--------------------------------------------------------------------------------------
void CALLBACK OnMouse( bool bLeftButtonDown, bool bRightButtonDown, bool bMiddleButtonDown, 
                      bool bSideButton1Down, bool bSideButton2Down, int nMouseWheelDelta, 
                      int xPos, int yPos, void* pUserContext )
{
}


//--------------------------------------------------------------------------------------
// Call if device was removed.  Return true to find a new device, false to quit
//--------------------------------------------------------------------------------------
bool CALLBACK OnDeviceRemoved( void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Initialize everything and go into a render loop
//--------------------------------------------------------------------------------------

int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nShowCmd )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // DXUT will create and use the best device (either D3D9 or D3D10) 
    // that is available on the system depending on which D3D callbacks are set below

    // Set general DXUT callbacks
    DXUTSetCallbackFrameMove( OnFrameMove );
    DXUTSetCallbackMouse( OnMouse );
    DXUTSetCallbackMsgProc( WndProc );
    DXUTSetCallbackKeyboard( KeyboardProc );
    DXUTSetCallbackDeviceRemoved( OnDeviceRemoved );

    // Set the D3D10 DXUT callbacks. Remove these sets if the app doesn't need to support D3D10
    DXUTSetCallbackD3D10DeviceAcceptable( IsD3D10DeviceAcceptable );
    DXUTSetCallbackD3D10DeviceCreated( OnD3D10CreateDevice );
    DXUTSetCallbackD3D10SwapChainResized( OnD3D10ResizedSwapChain );
    DXUTSetCallbackD3D10FrameRender( OnD3D10FrameRender );
    DXUTSetCallbackD3D10SwapChainReleasing( OnD3D10ReleasingSwapChain );
    DXUTSetCallbackD3D10DeviceDestroyed( OnD3D10DestroyDevice );

    InitGUI();

    DXUTInit( true, false, NULL );
    DXUTSetCursorSettings( true, true );
    DXUTCreateWindow( L"DX10 Feature Demo" );
    DXUTCreateDevice( true, g_WindowSizeX, g_WindowSizeY );
    DXUTMainLoop();

    // Perform any application-level cleanup here

    return DXUTGetExitCode();
}